# Workflow adapted from the SegOptim R package documentation by Joo Gonalves: https://segoptim.bitbucket.io/docs/index.html
library(raster)
library(randomForest)
library(SegOptim)
library(sf)
library(rgdal)
library(dplyr)
otbPath <- 'path to Orfeo ToolBox bin folder'

##################################Set min and max parameters (spatial range radius, spectral range radius, minimum object size )
OTB_min <- c( , , ) 
OTB_max <- c( , , )
##################################


##################################optimization inputs: subset of mosaic, training for mosaic, laplacian of subset for segmentation

# Classification features- raster stack
rstClassifFeatures <- stack('raster stack to be used as classification features')


# Train data- raster layer
rstTrainData <- raster('raster that delineates training areas (e.g., coded as integers)')


#path to segmentation features
segmFeatures <- ('path to Laplacian filter') #can also use mosaic as segmentation layer
###################################




###################################Segmentation optimization
gaOptim <- gaOptimizeSegmentationParams(  rstFeatures = rstClassifFeatures,
                                          trainData   = rstTrainData,
                                          # // Segmentation parameters ---
                                          segmentMethod = "OTB_LSMS",
                                          inputRstPath  = segmFeatures,
                                          otbBinPath    = otbPath, 
                                          verbose = FALSE,
                                          # // End segmentation parameters ---
                                          trainThresh   = 0.75,
                                          segmStatsFuns = c("mean"),
                                          classificationMethod = "RF",
                                          classificationMethodParams = NULL,
                                          balanceTrainData = FALSE,
                                          balanceMethod = "ubOver",
                                          evalMethod = "10FCV",
                                          evalMetric = "Kappa",
                                          minTrainCases = 20,
                                          minCasesByClassTrain = 5,
                                          minCasesByClassTest = 5,
                                          lower = OTB_min,
                                          upper = OTB_max,
                                          popSize = 20, 
                                          pcrossover = 0.8, 
                                          pmutation = 0.2, 
                                          maxiter = 5, 
                                          run = 20,
                                          keepBest = TRUE)

print(gaOptim@fitnessValue)
print(gaOptim@solution)
##########################################






############################################segmentation of entire mosaic



outSegmRst.path <- "output path for segmentation result"
inputRstPath_lap_OTB <- ('path to Laplacian filter for entire scene')
outSegmRst <- segmentation_OTB_LSMS(
  # Input raster with features/bands to segment. In this case, the Laplacian filter is used
  inputRstPath = inputRstPath_lap_OTB, 
  # Input parameters from optimization procedure
  SpectralRange = , 
  SpatialRange = , 
  MinSize = ,
  # Output
  outputSegmRst = outSegmRst.path,
  verbose = TRUE,
  otbBinPath = otbPath,
  lsms_maxiter = 5)
##############################################


##############################################remove water objects before classifying habitats

segmRst <- raster('path to segmentation file')
segmRst[segmRst == 1] <- NA #set water objects to null


writeRaster(segmRst, 'output file path for segmentation excluding water objects')
###############################################



##############################################paths for classification
dsm <- raster('path to dsm for entire scene')
trainDataRst <- raster('training areas raster for entire scene')
trainDataRst[trainDataRst < 1]<- NA #used if no data values are appearing as classes

#generate one raster that includes DSM and mosaic
mosaic <- stack('raster stack representing orthomosaic for entire scene')
mosaic <- dropLayer(mosaic, 4) #dropped alpha layer

classificationFeatures <- stack(mosaic, dsm) #stack the orthomosaic and dsm into one raster layer


###############################################




###############################################prep calibration
segmRst_water_removed <- ('path to segmented raster with water removed')


calData <- prepareCalData(rstSegm = segmRst_water_removed, 
                          trainData = trainDataRst, 
                          rstFeatures = classificationFeatures, 
                          thresh = 0.75, 
                          funs = c('sd',"mean"), 
                          minImgSegm = 30, 
                          verbose = TRUE
)
#################################################


#################################################calibration
classifObj <- calibrateClassifier( calData                    = calData,
                                   classificationMethod       = "RF",
                                   balanceTrainData           = FALSE,
                                   balanceMethod              = "ubOver",
                                   evalMethod                 = "10FCV",
                                   evalMetric                 = "Kappa",
                                   minTrainCases              = 30,
                                   minCasesByClassTrain       = 10,
                                   minCasesByClassTest        = 5,
                                   runFullCalibration         = TRUE)


#################################################



#################################################predict class label for objects

outClassRst.path <- ('output path for classification')
segmentation <- raster('path to segmentation with water removed')

rstPredSegmRF <- predictSegments(classifierObj = classifObj, 
                                 calData = calData, 
                                 rstSegm = segmentation, 
                                 predictFor = "all", 
                                 filename = outClassRst.path)
print(rstPredSegmRF)


classifObj[["ClassObj"]][["FULL"]][["importance"]] #displays the importance of variables